#include "graph.h"
#include "ui_graph.h"

#include <QSettings>
#include <QDebug>
#include <QMessageBox>
#include <QKeyEvent>
#include <QMouseEvent>
#include <QImageWriter>
#include <QSvgGenerator>
#include <QPrinter>
#include <QPalette>
#include <QFileDialog>
#include <QFont>
#include <qwt_legend.h>
#include <qwt_plot_canvas.h>
#include <qwt_scale_widget.h>
#include <qwt_symbol.h>

Graph::Graph(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::Graph)
{
    ui->setupUi(this);
    this->setWindowFlags(Qt::WindowMinimizeButtonHint | Qt::WindowMaximizeButtonHint | Qt::WindowCloseButtonHint );


    // Lecture et initialisation des paramètres par défaut
    MaxNoise=16;
    MinNoise=0;
    MaxMaxRate=5000;
    MinMaxRate=0;
    MinCRC=0.5;
    MaxCRC=100;
    MinHEC=0.5;
    MaxHEC=100;
    MinFEC=50;
    MaxFEC=50000;
    ReculMinute=5;
    PlageHeure=3;
    TempsMaxHeure=24;
    DecHistoSec=15;
    PenCurve=2;
    PenHisto=2;
    PenSynch=2;
    ExportScale=3;
    ExportResol=85;
    DiapoErreurs=true;
    DiapoCourbes=true;
    SSID="";
    TpsMajor=3600*1000/2;   // 30mn recalcule en ms pour Qwt
    NbTpsMed=2;             // 30mn / 2 = 15mn
    NbTpsMin=6;             // 30mn / 6 = 5mn
    TempsArret=5*60;        // 5mn
    TpsHeureFormat  = "hh:mm";
    TpsDateFormat   = "dd/MM";
    TpsSeparFormat  = "\n";     // "\n"=saut de ligne
    RapPageOrient   = QPrinter::Landscape;
    NbJrSynch = 20;

    // lecture du fichier ini
    QSettings settings(QApplication::applicationDirPath() + "/livebox.ini", QSettings::IniFormat);
    this->setGeometry(settings.value("GRAPH/X").toInt(),settings.value("GRAPH/Y").toInt(),settings.value("GRAPH/Width").toInt(),settings.value("GRAPH/Height").toInt());
    QList<int> SSize;
    SSize.clear();
    SSize.append(settings.value("GRAPH/Size0").toInt());
    SSize.append(settings.value("GRAPH/Size1").toInt());
    ui->splitterGraph->setSizes(SSize);
    SSID            =settings.value("LIVEBOX/SSID").toString();
    MaxNoise        =settings.value("COURBES/MaxNoise").toDouble();
    MinNoise        =settings.value("COURBES/MinNoise").toDouble();
    MaxMaxRate      =settings.value("COURBES/MaxMaxRate").toDouble();
    MinMaxRate      =settings.value("COURBES/MinMaxRate").toDouble();
    PasMaxRate      =settings.value("COURBES/PasMaxRate").toDouble();
    MaxCRC          =settings.value("COURBES/MaxCRC").toDouble();
    MinCRC          =settings.value("COURBES/MinCRC").toDouble();
    MaxHEC          =settings.value("COURBES/MaxHEC").toDouble();
    MinHEC          =settings.value("COURBES/MinHEC").toDouble();
    MaxFEC          =settings.value("COURBES/MaxFEC").toDouble();
    MinFEC          =settings.value("COURBES/MinFEC").toDouble();
    ReculMinute     =settings.value("COURBES/ReculMinute").toDouble();
    PlageHeure      =settings.value("COURBES/PlageHeure").toDouble();
    TempsMaxHeure   =settings.value("COURBES/TempsMaxHeure").toDouble();
    DecHistoSec     =settings.value("COURBES/DecHistoSec").toDouble();
    PenCurve        =settings.value("COURBES/PenCurve").toDouble();
    PenHisto        =settings.value("COURBES/PenHisto").toDouble();
    PenSynch        =settings.value("COURBES/PenSynch").toDouble();
    ExportScale     =settings.value("COURBES/ExportScale").toDouble();
    ExportResol     =settings.value("COURBES/ExportResol").toInt();
    DiapoErreurs    =settings.value("COURBES/DiapoErreurs").toBool();
    DiapoCourbes    =settings.value("COURBES/DiapoCourbes").toBool();
    EchDate         =settings.value("FORMAT/EchDate").toBool();
    EchHeure        =settings.value("FORMAT/EchHeure").toBool();
    TpsDateFormat   =settings.value("FORMAT/TpsDateFormat").toString();
    TpsHeureFormat  =settings.value("FORMAT/TpsHeureFormat").toString();
    TpsSeparFormat  =settings.value("FORMAT/TpsSeparFormat").toString();
    RapPageOrient   =settings.value("FORMAT/RapPageOrient").toInt();
    NbJrSynch       =settings.value("COURBES/NbJrSynch").toDouble();
    ui->tabErreurs->setCurrentIndex(settings.value("GRAPH/IndexErreurs").toInt());
    ui->tabCourbes->setCurrentIndex(settings.value("GRAPH/IndexCourbes").toInt());
    ui->checkNoisePdf->setChecked(settings.value("GRAPH/NoisePdf").toBool());
    ui->checkDebitMaxPdf->setChecked(settings.value("GRAPH/DebitMaxPdf").toBool());
    ui->checkCrcPdf->setChecked(settings.value("GRAPH/CrcPdf").toBool());
    ui->checkHecPdf->setChecked(settings.value("GRAPH/HecPdf").toBool());
    ui->checkFecPdf->setChecked(settings.value("GRAPH/FecPdf").toBool());


    // détermination de la base de temps des graphiques
    QDateTime qdtNow=QDateTime::currentDateTime();
    QwtDate qwtNow;
    double dNow=qwtNow.toDouble(qdtNow);
    // version originale
//    double Deb=dNow-ReculMinute*60*1000;              // début = heure de lancement moins ReculMinute
//    double Fin=Deb+PlageHeure*3600*1000;
//    Debut=Deb;
    // version modifiée pour graphique avec dernier relevé à droite
    Debut       =   dNow;                               // sera la base quand le début de feuille sera plein
    Dernier     =   Debut;
    BordDroit   =   dNow+ReculMinute*60*1000;
    double Fin  =   BordDroit;
    double Deb  =   Fin-PlageHeure*3600*1000;
    XtpsMax=Fin;
    XtpsMin=Deb;

    // initialisation scroll de temps
    Offset=Dernier;
    ui->scrollTemps->setMaximum((Dernier-Debut)/60000);
    ui->scrollTemps->setValue((Dernier-Offset)/60000);
    ui->scrollTemps->setEnabled(!ui->checkAvanceAuto->isChecked());

    // calcul des graduations de temps en fonction de la plage de vue
    TpsMajor = 0.5 * 3600 * 1000;       // 30 mn
    NbTpsMed = 2;                       // 15 mn
    NbTpsMin = 6;                       //  5 mn
    CoefPen  = 1;
    if (PlageHeure > 6) {
        TpsMajor = 1 * 3600 * 1000;     //  1 h
        NbTpsMed = 4;                   // 15 mn
        NbTpsMin = 12;                  //  5 mn
        CoefPen  = 0.75;
    }
    if (PlageHeure > 12) {
        TpsMajor = 2 * 3600 * 1000;     //  2 h
        NbTpsMed = 4;                   // 30 mn
        NbTpsMin = 8;                   // 15 mn
        CoefPen  = 0.5;
    }
    // Définition des échelles horizontales pour les temps
    TpsFormat       = "";
    if (EchDate)    TpsFormat=TpsSeparFormat+TpsDateFormat;
    if (EchHeure)   TpsFormat=TpsHeureFormat+TpsFormat;
    // pour courbe marges de bruit
    qwtScaleDrawNoise = new QwtDateScaleDraw();
    qwtScaleDrawNoise->setDateFormat(QwtDate::Millisecond,TpsFormat);
    qwtScaleDrawNoise->setDateFormat(QwtDate::Second,TpsFormat);
    qwtScaleDrawNoise->setDateFormat(QwtDate::Minute,TpsFormat);
    qwtScaleDrawNoise->setDateFormat(QwtDate::Hour,TpsFormat);
    qwtScaleDrawNoise->setDateFormat(QwtDate::Day,TpsFormat);
    qwtScaleDrawNoise->setDateFormat(QwtDate::Month,TpsFormat);
    qwtScaleDrawNoise->setDateFormat(QwtDate::Year,TpsFormat);
    qwtScaleDrawNoise->setLabelAlignment(Qt::AlignHCenter|Qt::AlignBottom);
    // pour courbe débits maxi
    qwtScaleDrawMaxRate = new QwtDateScaleDraw();
    qwtScaleDrawMaxRate->setDateFormat(QwtDate::Millisecond,TpsFormat);
    qwtScaleDrawMaxRate->setDateFormat(QwtDate::Second,TpsFormat);
    qwtScaleDrawMaxRate->setDateFormat(QwtDate::Minute,TpsFormat);
    qwtScaleDrawMaxRate->setDateFormat(QwtDate::Hour,TpsFormat);
    qwtScaleDrawMaxRate->setDateFormat(QwtDate::Day,TpsFormat);
    qwtScaleDrawMaxRate->setDateFormat(QwtDate::Month,TpsFormat);
    qwtScaleDrawMaxRate->setDateFormat(QwtDate::Year,TpsFormat);
    qwtScaleDrawMaxRate->setLabelAlignment(Qt::AlignHCenter|Qt::AlignBottom);
    // pour courbe nombre de CRC
    qwtScaleDrawCRC = new QwtDateScaleDraw();
    qwtScaleDrawCRC->setDateFormat(QwtDate::Millisecond,TpsFormat);
    qwtScaleDrawCRC->setDateFormat(QwtDate::Second,TpsFormat);
    qwtScaleDrawCRC->setDateFormat(QwtDate::Minute,TpsFormat);
    qwtScaleDrawCRC->setDateFormat(QwtDate::Hour,TpsFormat);
    qwtScaleDrawCRC->setDateFormat(QwtDate::Day,TpsFormat);
    qwtScaleDrawCRC->setDateFormat(QwtDate::Month,TpsFormat);
    qwtScaleDrawCRC->setDateFormat(QwtDate::Year,TpsFormat);
    qwtScaleDrawCRC->setLabelAlignment(Qt::AlignHCenter|Qt::AlignBottom);
    // pour courbe nombre de FEC
    qwtScaleDrawFEC = new QwtDateScaleDraw();
    qwtScaleDrawFEC->setDateFormat(QwtDate::Millisecond,TpsFormat);
    qwtScaleDrawFEC->setDateFormat(QwtDate::Second,TpsFormat);
    qwtScaleDrawFEC->setDateFormat(QwtDate::Minute,TpsFormat);
    qwtScaleDrawFEC->setDateFormat(QwtDate::Hour,TpsFormat);
    qwtScaleDrawFEC->setDateFormat(QwtDate::Day,TpsFormat);
    qwtScaleDrawFEC->setDateFormat(QwtDate::Month,TpsFormat);
    qwtScaleDrawFEC->setDateFormat(QwtDate::Year,TpsFormat);
    qwtScaleDrawFEC->setLabelAlignment(Qt::AlignHCenter|Qt::AlignBottom);
    // pour courbe nombre de HEC
    qwtScaleDrawHEC = new QwtDateScaleDraw();
    qwtScaleDrawHEC->setDateFormat(QwtDate::Millisecond,TpsFormat);
    qwtScaleDrawHEC->setDateFormat(QwtDate::Second,TpsFormat);
    qwtScaleDrawHEC->setDateFormat(QwtDate::Minute,TpsFormat);
    qwtScaleDrawHEC->setDateFormat(QwtDate::Hour,TpsFormat);
    qwtScaleDrawHEC->setDateFormat(QwtDate::Day,TpsFormat);
    qwtScaleDrawHEC->setDateFormat(QwtDate::Month,TpsFormat);
    qwtScaleDrawHEC->setDateFormat(QwtDate::Year,TpsFormat);
    qwtScaleDrawHEC->setLabelAlignment(Qt::AlignHCenter|Qt::AlignBottom);

    // Définition des grilles
    // pour le graphique marge de bruit
    GridNoise = new QwtPlotGrid();
    GridNoise->setMajorPen( Qt::darkGray, 0.0, Qt::SolidLine );
    GridNoise->setMinorPen( Qt::lightGray, 0.0, Qt::SolidLine);
    GridNoise->enableX(true);
    GridNoise->enableY(true);
    GridNoise->enableXMin(true);
    GridNoise->enableYMin(false);
    GridNoise->attach(ui->qwtPlotNoise);
    // pour le graphique débits maxi
    GridMaxRate = new QwtPlotGrid();
    GridMaxRate->setMajorPen( Qt::darkGray, 0.0, Qt::SolidLine );
    GridMaxRate->setMinorPen( Qt::lightGray, 0.0, Qt::SolidLine);
    GridMaxRate->enableX(true);
    GridMaxRate->enableY(true);
    GridMaxRate->enableXMin(true);
    GridMaxRate->enableYMin(true);
    GridMaxRate->attach(ui->qwtPlotMaxRate);
    // pour le graphique nombre de CRC
    GridCRC = new QwtPlotGrid();
    GridCRC->setMajorPen( Qt::darkGray, 0.0, Qt::SolidLine );
    GridCRC->setMinorPen( Qt::lightGray, 0.0, Qt::SolidLine);
    GridCRC->enableX(true);
    GridCRC->enableY(true);
    GridCRC->enableXMin(true);
    GridCRC->enableYMin(true);
    GridCRC->attach(ui->qwtPlotCRC);
    // pour le graphique nombre de HEC
    GridHEC = new QwtPlotGrid();
    GridHEC->setMajorPen( Qt::darkGray, 0.0, Qt::SolidLine );
    GridHEC->setMinorPen( Qt::lightGray, 0.0, Qt::SolidLine);
    GridHEC->enableX(true);
    GridHEC->enableY(true);
    GridHEC->enableXMin(true);
    GridHEC->enableYMin(true);
    GridHEC->attach(ui->qwtPlotHEC);
    // pour le graphique nombre de FEC
    GridFEC = new QwtPlotGrid();
    GridFEC->setMajorPen( Qt::darkGray, 0.0, Qt::SolidLine );
    GridFEC->setMinorPen( Qt::lightGray, 0.0, Qt::SolidLine);
    GridFEC->enableX(true);
    GridFEC->enableY(true);
    GridFEC->enableXMin(true);
    GridFEC->enableYMin(true);
    GridFEC->attach(ui->qwtPlotFEC);

    // Construction des graphiques et de leurs échelles
    // graphique marge de bruit
    ui->qwtPlotNoise->enableAxis(QwtPlot::yLeft,true);
    ui->qwtPlotNoise->setAxisScale(QwtPlot::yLeft,MinNoise,MaxNoise,1);
    ui->qwtPlotNoise->setAxisMaxMinor(QwtPlot::yLeft,10);
    ui->qwtPlotNoise->setAxisMaxMajor(QwtPlot::yLeft,2);
    ui->qwtPlotNoise->setAxisScaleEngine(QwtPlot::xBottom,new QwtDateScaleEngine());
    ui->qwtPlotNoise->setAxisScale(QwtPlot::xBottom,Deb,Fin,TpsMajor);
    ui->qwtPlotNoise->setAxisMaxMajor(QwtPlot::xBottom,NbTpsMed);
    ui->qwtPlotNoise->setAxisMaxMinor(QwtPlot::xBottom,NbTpsMin);
    ui->qwtPlotNoise->setAxisScaleDraw(QwtPlot::xBottom,qwtScaleDrawNoise);
    // graphique marge de débit maxi
    ui->qwtPlotMaxRate->enableAxis(QwtPlot::yLeft,true);
    ui->qwtPlotMaxRate->setAxisScale(QwtPlot::yLeft,MinMaxRate,MaxMaxRate,PasMaxRate);
    ui->qwtPlotMaxRate->setAxisMaxMinor(QwtPlot::yLeft,10);
    ui->qwtPlotMaxRate->setAxisMaxMajor(QwtPlot::yLeft,2);
    ui->qwtPlotMaxRate->setAxisScaleEngine(QwtPlot::xBottom,new QwtDateScaleEngine());
    ui->qwtPlotMaxRate->setAxisScale(QwtPlot::xBottom,Deb,Fin,TpsMajor);
    ui->qwtPlotMaxRate->setAxisMaxMajor(QwtPlot::xBottom,NbTpsMed);
    ui->qwtPlotMaxRate->setAxisMaxMinor(QwtPlot::xBottom,NbTpsMin);
    ui->qwtPlotMaxRate->setAxisScaleDraw(QwtPlot::xBottom,qwtScaleDrawMaxRate);
    // graphique nombre de CRC ==> echelle logarithmique
    ui->qwtPlotCRC->setAxisScaleEngine(QwtPlot::yLeft,new QwtLogScaleEngine());
    ui->qwtPlotCRC->setAxisScale(QwtPlot::yLeft,MinCRC,MaxCRC);
    ui->qwtPlotCRC->setAxisMaxMinor(QwtPlot::yLeft,10);
    ui->qwtPlotCRC->setAxisScaleEngine(QwtPlot::xBottom,new QwtDateScaleEngine());
    ui->qwtPlotCRC->setAxisScale(QwtPlot::xBottom,Deb,Fin,TpsMajor);
    ui->qwtPlotCRC->setAxisMaxMajor(QwtPlot::xBottom,NbTpsMed);
    ui->qwtPlotCRC->setAxisMaxMinor(QwtPlot::xBottom,NbTpsMin);
    ui->qwtPlotCRC->setAxisScaleDraw(QwtPlot::xBottom,qwtScaleDrawCRC);
    // graphique nombre de HEC ==> echelle logarithmique
    ui->qwtPlotHEC->setAxisScaleEngine(QwtPlot::yLeft,new QwtLogScaleEngine());
    ui->qwtPlotHEC->setAxisScale(QwtPlot::yLeft,MinHEC,MaxHEC);
    ui->qwtPlotHEC->setAxisMaxMinor(QwtPlot::yLeft,10);
    ui->qwtPlotHEC->setAxisScaleEngine(QwtPlot::xBottom,new QwtDateScaleEngine());
    ui->qwtPlotHEC->setAxisScale(QwtPlot::xBottom,Deb,Fin,TpsMajor);
    ui->qwtPlotHEC->setAxisMaxMajor(QwtPlot::xBottom,NbTpsMed);
    ui->qwtPlotHEC->setAxisMaxMinor(QwtPlot::xBottom,NbTpsMin);
    ui->qwtPlotHEC->setAxisScaleDraw(QwtPlot::xBottom,qwtScaleDrawHEC);
    // graphique nombre de FEC ==> echelle logarithmique
    ui->qwtPlotFEC->setAxisScaleEngine(QwtPlot::yLeft,new QwtLogScaleEngine());
    ui->qwtPlotFEC->setAxisScale(QwtPlot::yLeft,MinFEC,MaxFEC);
    ui->qwtPlotFEC->setAxisMaxMinor(QwtPlot::yLeft,10);
    ui->qwtPlotFEC->setAxisScaleEngine(QwtPlot::xBottom,new QwtDateScaleEngine());
    ui->qwtPlotFEC->setAxisScale(QwtPlot::xBottom,Deb,Fin,TpsMajor);
    ui->qwtPlotFEC->setAxisMaxMajor(QwtPlot::xBottom,NbTpsMed);
    ui->qwtPlotFEC->setAxisMaxMinor(QwtPlot::xBottom,NbTpsMin);
    ui->qwtPlotFEC->setAxisScaleDraw(QwtPlot::xBottom,qwtScaleDrawFEC);

    // Définition des courbes et association aux graphiques
    // synchronisations sur marges
    CurveSynchNoise = new QwtPlotCurve();
    CurveSynchNoise->setTitle("Synchro DSL");
    CurveSynchNoise->setStyle(QwtPlotCurve::Sticks);
    CurveSynchNoise->setBaseline(0.1);
    CurveSynchNoise->setYAxis(QwtPlot::yLeft);
    CurveSynchNoise->setPen(Qt::magenta,PenSynch,Qt::DashLine);
    CurveSynchNoise->attach(ui->qwtPlotNoise);
    // marge de bruit descendante
    CurveDownNoise = new QwtPlotCurve();
    CurveDownNoise->setTitle("DOWNLOAD");
    CurveDownNoise->setYAxis(QwtPlot::yLeft);
    CurveDownNoise->setPen(Qt::blue,CoefPen*PenCurve,Qt::DotLine);
    CurveDownNoise->attach(ui->qwtPlotNoise);
    // marge de bruit montante
    CurveUpNoise = new QwtPlotCurve();
    CurveUpNoise->setTitle("UPLOAD");
    CurveUpNoise->setYAxis(QwtPlot::yLeft);
    CurveUpNoise->setPen(Qt::red,CoefPen*PenCurve,Qt::DotLine);
    CurveUpNoise->attach(ui->qwtPlotNoise);
    // synchronisations sur débits maxi
    CurveSynchMaxRate = new QwtPlotCurve();
    CurveSynchMaxRate->setTitle("Synchro DSL");
    CurveSynchMaxRate->setStyle(QwtPlotCurve::Sticks);
    CurveSynchMaxRate->setBaseline(0.1);
    CurveSynchMaxRate->setYAxis(QwtPlot::yLeft);
    CurveSynchMaxRate->setPen(Qt::magenta,PenSynch,Qt::DashLine);
    CurveSynchMaxRate->attach(ui->qwtPlotMaxRate);
    // débit descendant maxi
    CurveDownMaxRate = new QwtPlotCurve();
    CurveDownMaxRate->setTitle("MAXI DOWN");
    CurveDownMaxRate->setYAxis(QwtPlot::yLeft);
    CurveDownMaxRate->setPen(Qt::darkBlue,CoefPen*PenCurve,Qt::DotLine);
    CurveDownMaxRate->attach(ui->qwtPlotMaxRate);
    // débit montant maxi
    CurveUpMaxRate = new QwtPlotCurve();
    CurveUpMaxRate->setTitle("MAXI UP");
    CurveUpMaxRate->setYAxis(QwtPlot::yLeft);
    CurveUpMaxRate->setPen(Qt::darkRed,CoefPen*PenCurve,Qt::DotLine);
    CurveUpMaxRate->attach(ui->qwtPlotMaxRate);
    // débit DSL descendant
    CurveDownDslRate = new QwtPlotCurve();
    CurveDownDslRate->setTitle("DSL DOWN");
    CurveDownDslRate->setYAxis(QwtPlot::yLeft);
    CurveDownDslRate->setPen(Qt::blue,CoefPen*PenCurve,Qt::SolidLine);
    CurveDownDslRate->attach(ui->qwtPlotMaxRate);
    // débit DSL montant
    CurveUpDslRate = new QwtPlotCurve();
    CurveUpDslRate->setTitle("DSL UP");
    CurveUpDslRate->setYAxis(QwtPlot::yLeft);
    CurveUpDslRate->setPen(Qt::red,CoefPen*PenCurve,Qt::SolidLine);
    CurveUpDslRate->attach(ui->qwtPlotMaxRate);
    // synchronisation sur CRC
    CurveSynchCRC = new QwtPlotCurve();
    CurveSynchCRC->setTitle("Synchro DSL");
    CurveSynchCRC->setStyle(QwtPlotCurve::Sticks);
    CurveSynchCRC->setBaseline(0.1);
    CurveSynchCRC->setYAxis(QwtPlot::yLeft);
    CurveSynchCRC->setPen(Qt::magenta,PenSynch,Qt::DashLine);
    CurveSynchCRC->attach(ui->qwtPlotCRC);
    // erreurs CRC en échelle logarithmique
    CurveLogCRC = new QwtPlotCurve();
    CurveLogCRC->setTitle("CRC");
    CurveLogCRC->setStyle(QwtPlotCurve::Sticks);
    CurveLogCRC->setBaseline(0.1);
    CurveLogCRC->setYAxis(QwtPlot::yLeft);
    CurveLogCRC->setPen(Qt::blue,CoefPen*PenHisto,Qt::SolidLine);
    CurveLogCRC->attach(ui->qwtPlotCRC);
    // erreurs ATU-C CRC en échelle logarithmique
    CurveLogATUCCRC = new QwtPlotCurve();
    CurveLogATUCCRC->setTitle("ATU-C CRC");
    CurveLogATUCCRC->setStyle(QwtPlotCurve::Sticks);
    CurveLogATUCCRC->setBaseline(0.1);
    CurveLogATUCCRC->setYAxis(QwtPlot::yLeft);
    CurveLogATUCCRC->setPen(Qt::red,CoefPen*PenHisto,Qt::SolidLine);
    CurveLogATUCCRC->attach(ui->qwtPlotCRC);
    // synchronisation sur HEC
    CurveSynchHEC = new QwtPlotCurve();
    CurveSynchHEC->setTitle("Synchro DSL");
    CurveSynchHEC->setStyle(QwtPlotCurve::Sticks);
    CurveSynchHEC->setBaseline(0.1);
    CurveSynchHEC->setYAxis(QwtPlot::yLeft);
    CurveSynchHEC->setPen(Qt::magenta,PenSynch,Qt::DashLine);
    CurveSynchHEC->attach(ui->qwtPlotHEC);
    // erreurs HEC en échelle logarithmique
    CurveLogHEC = new QwtPlotCurve();
    CurveLogHEC->setTitle("HEC");
    CurveLogHEC->setStyle(QwtPlotCurve::Sticks);
    CurveLogHEC->setBaseline(0.1);
    CurveLogHEC->setYAxis(QwtPlot::yLeft);
    CurveLogHEC->setPen(QColor(80,110,180,255),CoefPen*PenHisto,Qt::SolidLine);
    CurveLogHEC->attach(ui->qwtPlotHEC);
    // erreurs ATU-C HEC en échelle logarithmique
    CurveLogATUCHEC = new QwtPlotCurve();
    CurveLogATUCHEC->setTitle("ATU-C HEC");
    CurveLogATUCHEC->setStyle(QwtPlotCurve::Sticks);
    CurveLogATUCHEC->setBaseline(0.1);
    CurveLogATUCHEC->setYAxis(QwtPlot::yLeft);
    CurveLogATUCHEC->setPen(QColor(220,40,70,255),CoefPen*PenHisto,Qt::SolidLine); // orange
    CurveLogATUCHEC->attach(ui->qwtPlotHEC);
    // synchronisation sur FEC
    CurveSynchFEC = new QwtPlotCurve();
    CurveSynchFEC->setTitle("Synchro DSL");
    CurveSynchFEC->setStyle(QwtPlotCurve::Sticks);
    CurveSynchFEC->setBaseline(0.1);
    CurveSynchFEC->setYAxis(QwtPlot::yLeft);
    CurveSynchFEC->setPen(Qt::magenta,PenSynch,Qt::DashLine);
    CurveSynchFEC->attach(ui->qwtPlotFEC);
    // erreurs FEC en échelle logarithmique
    CurveLogFEC = new QwtPlotCurve();
    CurveLogFEC->setTitle("FEC");
    CurveLogFEC->setStyle(QwtPlotCurve::Sticks);
    CurveLogFEC->setBaseline(0.1);
    CurveLogFEC->setYAxis(QwtPlot::yLeft);
    CurveLogFEC->setPen(Qt::darkGreen,CoefPen*PenHisto,Qt::SolidLine);
    CurveLogFEC->attach(ui->qwtPlotFEC);
    // erreurs ATU-C FEC en échelle logarithmique
    CurveLogATUCFEC = new QwtPlotCurve();
    CurveLogATUCFEC->setTitle("ATU-C FEC");
    CurveLogATUCFEC->setStyle(QwtPlotCurve::Sticks);
    CurveLogATUCFEC->setBaseline(0.1);
    CurveLogATUCFEC->setYAxis(QwtPlot::yLeft);
    CurveLogATUCFEC->setPen(QColor(255,192,0,255),CoefPen*PenHisto,Qt::SolidLine); // orange
    CurveLogATUCFEC->attach(ui->qwtPlotFEC);

    // Ajout des légendes
    ui->qwtPlotNoise->insertLegend(new QwtLegend(), QwtPlot::RightLegend);
    ui->qwtPlotMaxRate->insertLegend(new QwtLegend(), QwtPlot::RightLegend);
    ui->qwtPlotCRC->insertLegend(new QwtLegend(), QwtPlot::RightLegend);
    ui->qwtPlotHEC->insertLegend(new QwtLegend(), QwtPlot::RightLegend);
    ui->qwtPlotFEC->insertLegend(new QwtLegend(), QwtPlot::RightLegend);

    // Pickers pour selection de point de temps et repérage
    PickerNoise = new QwtPlotPicker(ui->qwtPlotNoise->canvas());
    PickerNoise->setStateMachine(new QwtPickerClickPointMachine);
    connect(PickerNoise,SIGNAL(selected(QPointF)),this,SLOT(on_Picker_Temps(QPointF)));
    PickerNoise->setEnabled(true);

    PickerMaxRate = new QwtPlotPicker(ui->qwtPlotMaxRate->canvas());
    PickerMaxRate->setStateMachine(new QwtPickerClickPointMachine);
    connect(PickerMaxRate,SIGNAL(selected(QPointF)),this,SLOT(on_Picker_Temps(QPointF)));
    PickerMaxRate->setEnabled(true);

    PickerCRC = new QwtPlotPicker(ui->qwtPlotCRC->canvas());
    PickerCRC->setStateMachine(new QwtPickerClickPointMachine);
    connect(PickerCRC,SIGNAL(selected(QPointF)),this,SLOT(on_Picker_Temps(QPointF)));
    PickerCRC->setEnabled(true);

    PickerHEC = new QwtPlotPicker(ui->qwtPlotHEC->canvas());
    PickerHEC->setStateMachine(new QwtPickerClickPointMachine);
    connect(PickerHEC,SIGNAL(selected(QPointF)),this,SLOT(on_Picker_Temps(QPointF)));
    PickerHEC->setEnabled(true);

    PickerFEC = new QwtPlotPicker(ui->qwtPlotFEC->canvas());
    PickerFEC->setStateMachine(new QwtPickerClickPointMachine);
    connect(PickerFEC,SIGNAL(selected(QPointF)),this,SLOT(on_Picker_Temps(QPointF)));
    PickerFEC->setEnabled(true);

    // EnventFilter pour intercepter les click droit dans les graphiques (voir slot Graph::eventFilter plus bas dans le code)
    ui->qwtPlotNoise->installEventFilter(this);
    ui->qwtPlotMaxRate->installEventFilter(this);
    ui->qwtPlotCRC->installEventFilter(this);
    ui->qwtPlotHEC->installEventFilter(this);
    ui->qwtPlotFEC->installEventFilter(this);

    // initialisations timer de diaporama
    DiapoTimer = new QTimer(this);
    connect(DiapoTimer, SIGNAL(timeout()), this, SLOT(DiapoTimerHandler()));
    DiapoTimer->stop();

//    ui->tabCourbes->setTabEnabled(GRAPH_SYNCHRO,false);
//    ui->tabCourbes->removeTab(GRAPH_SYNCHRO);
    InitTabSynchronisations();

}

Graph::~Graph()
{
    // sauvegarde de l'état de la fenetre "graphiques" dans le fichier ini
    QSettings settings(QApplication::applicationDirPath() + "/livebox.ini", QSettings::IniFormat);
    settings.setValue("GRAPH/Width",this->geometry().width());
    settings.setValue("GRAPH/Height",this->geometry().height());
    settings.setValue("GRAPH/X",this->geometry().x());
    settings.setValue("GRAPH/Y",this->geometry().y());
    settings.setValue("GRAPH/Size0",ui->splitterGraph->sizes()[0]);
    settings.setValue("GRAPH/Size1",ui->splitterGraph->sizes()[1]);
    settings.setValue("GRAPH/IndexCourbes",ui->tabCourbes->currentIndex());
    settings.setValue("GRAPH/IndexErreurs",ui->tabErreurs->currentIndex());
    settings.setValue("GRAPH/NoisePdf",ui->checkNoisePdf->isChecked());
    settings.setValue("GRAPH/DebitMaxPdf",ui->checkDebitMaxPdf->isChecked());
    settings.setValue("GRAPH/CrcPdf",ui->checkCrcPdf->isChecked());
    settings.setValue("GRAPH/HecPdf",ui->checkHecPdf->isChecked());
    settings.setValue("GRAPH/FecPdf",ui->checkFecPdf->isChecked());
    settings.setValue("COURBES/MaxNoise",MaxNoise);
    settings.setValue("COURBES/MinNoise",MinNoise);
    settings.setValue("COURBES/MaxMaxRate",MaxMaxRate);
    settings.setValue("COURBES/MinMaxRate",MinMaxRate);
    settings.setValue("COURBES/PasMaxRate",PasMaxRate);
    settings.setValue("COURBES/MaxCRC",MaxCRC);
    settings.setValue("COURBES/MinCRC",MinCRC);
    settings.setValue("COURBES/MaxHEC",MaxHEC);
    settings.setValue("COURBES/MinHEC",MinHEC);
    settings.setValue("COURBES/MaxFEC",MaxFEC);
    settings.setValue("COURBES/MinFEC",MinFEC);
    settings.setValue("COURBES/ReculMinute",ReculMinute);
    settings.setValue("COURBES/PlageHeure",PlageHeure);
    settings.setValue("COURBES/TempsMaxHeure",TempsMaxHeure);
    settings.setValue("COURBES/DecHistoSec",DecHistoSec);
    settings.setValue("COURBES/PenCurve",PenCurve);
    settings.setValue("COURBES/PenHisto",PenHisto);
    settings.setValue("COURBES/PenSynch",PenSynch);
    settings.setValue("COURBES/ExportScale",ExportScale);
    settings.setValue("COURBES/ExportResol",ExportResol);
    settings.setValue("COURBES/DiapoErreurs",DiapoErreurs);
    settings.setValue("COURBES/DiapoCourbes",DiapoCourbes);
    settings.setValue("COURBES/NbJrSynch",NbJrSynch);
    settings.setValue("FORMAT/EchDate",EchDate);
    settings.setValue("FORMAT/EchHeure",EchHeure);
    settings.setValue("FORMAT/TpsDateFormat",TpsDateFormat);
    settings.setValue("FORMAT/TpsHeureFormat",TpsHeureFormat);
    settings.setValue("FORMAT/TpsSeparFormat",TpsSeparFormat);
    settings.setValue("FORMAT/RapPageOrient",RapPageOrient);

    // destruction des objets définis par pointeurs
    delete DiapoTimer;

    delete CurveUpNoise;
    delete CurveDownNoise;
    delete CurveLogCRC;
    delete CurveLogATUCCRC;
    delete CurveLogHEC;
    delete CurveLogATUCHEC;
    delete CurveLogFEC;
    delete CurveLogATUCFEC;
    delete CurveSynchNoise;
    delete CurveSynchCRC;
    delete CurveSynchHEC;
    delete CurveSynchFEC;
    delete GridNoise;
    delete GridCRC;
    delete GridHEC;
    delete GridFEC;
    delete PickerNoise;
    delete PickerCRC;
    delete PickerHEC;
    delete PickerFEC;
    delete CurveSynchMaxRate;
    delete CurveDownMaxRate;
    delete CurveUpMaxRate;
    delete CurveDownDslRate;
    delete CurveUpDslRate;
    delete CurveSynchronisations;
    delete CurveReboot;
    delete CurveConnect;
    delete GridMaxRate;
    delete GridSynchronisations;
    delete PickerMaxRate;

    delete ui;
}

void Graph::UpDateScales()
{
    // echelles en X = temps donc la même chose pour chaque graphique
    // PlageHeure = période vue en affichage
    // Debut      = heure de début de relevé
    // Offset     = décalage utilisateur donné par le slider

    // version à défilement avec dernier relevé à droite
    double Deb;
    double Fin;
    QDateTime qdtNow=QDateTime::currentDateTime();
    QwtDate qwtNow;
    double dNow=qwtNow.toDouble(qdtNow);
    BordDroit=dNow+ReculMinute*60*1000;

    if (ui->checkAvanceAuto->isChecked()) {
        // on montre toujours le dernier relevé à droite
        Fin = BordDroit;
        Deb = Fin - PlageHeure*3600*1000;
        Offset = Dernier-Debut;                         // la barre de scroll reste à droite
        ui->scrollTemps->setMaximum(Offset/60000);
        ui->scrollTemps->setValue(Offset/60000);
    }
    else {
        // on reste sur la position choisie avec la barre scrollTemps
        Fin = Debut + Offset + ReculMinute*60*1000;
        Deb = Fin - PlageHeure*3600*1000;
    }
    XtpsMax=Fin;
    XtpsMin=Deb;

    // calcul des graduations de temps
    TpsMajor = 0.5 * 3600 * 1000;       // 30 mn
    NbTpsMed = 2;                       // 15 mn
    NbTpsMin = 6;                       //  5 mn
    CoefPen  = 1;
    if (PlageHeure > 6) {
        TpsMajor = 1 * 3600 * 1000;     //  1 h
        NbTpsMed = 4;                   // 15 mn
        NbTpsMin = 12;                  //  5 mn
        CoefPen  = 0.75;
    }
    if (PlageHeure > 12) {
        TpsMajor = 2 * 3600 * 1000;     //  2 h
        NbTpsMed = 4;                   // 30 mn
        NbTpsMin = 8;                   // 15 mn
        CoefPen  = 0.5;
    }

    // Définition des échelles horizontales pour les temps
    TpsFormat       = "";
    if (EchDate)    TpsFormat=TpsSeparFormat+TpsDateFormat;
    if (EchHeure)   TpsFormat=TpsHeureFormat+TpsFormat;
    // pour courbe marges de bruit
    qwtScaleDrawNoise->setDateFormat(QwtDate::Millisecond,TpsFormat);
    qwtScaleDrawNoise->setDateFormat(QwtDate::Second,TpsFormat);
    qwtScaleDrawNoise->setDateFormat(QwtDate::Minute,TpsFormat);
    qwtScaleDrawNoise->setDateFormat(QwtDate::Hour,TpsFormat);
    qwtScaleDrawNoise->setDateFormat(QwtDate::Day,TpsFormat);
    qwtScaleDrawNoise->setDateFormat(QwtDate::Month,TpsFormat);
    qwtScaleDrawNoise->setDateFormat(QwtDate::Year,TpsFormat);
    qwtScaleDrawNoise->setLabelAlignment(Qt::AlignHCenter|Qt::AlignBottom);
    // pour courbe débits maxi
    qwtScaleDrawMaxRate->setDateFormat(QwtDate::Millisecond,TpsFormat);
    qwtScaleDrawMaxRate->setDateFormat(QwtDate::Second,TpsFormat);
    qwtScaleDrawMaxRate->setDateFormat(QwtDate::Minute,TpsFormat);
    qwtScaleDrawMaxRate->setDateFormat(QwtDate::Hour,TpsFormat);
    qwtScaleDrawMaxRate->setDateFormat(QwtDate::Day,TpsFormat);
    qwtScaleDrawMaxRate->setDateFormat(QwtDate::Month,TpsFormat);
    qwtScaleDrawMaxRate->setDateFormat(QwtDate::Year,TpsFormat);
    qwtScaleDrawMaxRate->setLabelAlignment(Qt::AlignHCenter|Qt::AlignBottom);
    // pour courbe nombre de CRC
    qwtScaleDrawCRC->setDateFormat(QwtDate::Millisecond,TpsFormat);
    qwtScaleDrawCRC->setDateFormat(QwtDate::Second,TpsFormat);
    qwtScaleDrawCRC->setDateFormat(QwtDate::Minute,TpsFormat);
    qwtScaleDrawCRC->setDateFormat(QwtDate::Hour,TpsFormat);
    qwtScaleDrawCRC->setDateFormat(QwtDate::Day,TpsFormat);
    qwtScaleDrawCRC->setDateFormat(QwtDate::Month,TpsFormat);
    qwtScaleDrawCRC->setDateFormat(QwtDate::Year,TpsFormat);
    qwtScaleDrawCRC->setLabelAlignment(Qt::AlignHCenter|Qt::AlignBottom);
    // pour courbe nombre de FEC
    qwtScaleDrawFEC->setDateFormat(QwtDate::Millisecond,TpsFormat);
    qwtScaleDrawFEC->setDateFormat(QwtDate::Second,TpsFormat);
    qwtScaleDrawFEC->setDateFormat(QwtDate::Minute,TpsFormat);
    qwtScaleDrawFEC->setDateFormat(QwtDate::Hour,TpsFormat);
    qwtScaleDrawFEC->setDateFormat(QwtDate::Day,TpsFormat);
    qwtScaleDrawFEC->setDateFormat(QwtDate::Month,TpsFormat);
    qwtScaleDrawFEC->setDateFormat(QwtDate::Year,TpsFormat);
    qwtScaleDrawFEC->setLabelAlignment(Qt::AlignHCenter|Qt::AlignBottom);
    // pour courbe nombre de HEC
    qwtScaleDrawHEC->setDateFormat(QwtDate::Millisecond,TpsFormat);
    qwtScaleDrawHEC->setDateFormat(QwtDate::Second,TpsFormat);
    qwtScaleDrawHEC->setDateFormat(QwtDate::Minute,TpsFormat);
    qwtScaleDrawHEC->setDateFormat(QwtDate::Hour,TpsFormat);
    qwtScaleDrawHEC->setDateFormat(QwtDate::Day,TpsFormat);
    qwtScaleDrawHEC->setDateFormat(QwtDate::Month,TpsFormat);
    qwtScaleDrawHEC->setDateFormat(QwtDate::Year,TpsFormat);
    qwtScaleDrawHEC->setLabelAlignment(Qt::AlignHCenter|Qt::AlignBottom);

    ui->qwtPlotNoise->setAxisScale(QwtPlot::xBottom,Deb,Fin,TpsMajor);
    ui->qwtPlotNoise->setAxisMaxMajor(QwtPlot::xBottom,NbTpsMed);
    ui->qwtPlotNoise->setAxisMaxMinor(QwtPlot::xBottom,NbTpsMin);

    ui->qwtPlotMaxRate->setAxisScale(QwtPlot::xBottom,Deb,Fin,TpsMajor);
    ui->qwtPlotMaxRate->setAxisMaxMajor(QwtPlot::xBottom,NbTpsMed);
    ui->qwtPlotMaxRate->setAxisMaxMinor(QwtPlot::xBottom,NbTpsMin);

    ui->qwtPlotCRC->setAxisScale(QwtPlot::xBottom,Deb,Fin,TpsMajor);
    ui->qwtPlotCRC->setAxisMaxMajor(QwtPlot::xBottom,NbTpsMed);
    ui->qwtPlotCRC->setAxisMaxMinor(QwtPlot::xBottom,NbTpsMin);

    ui->qwtPlotHEC->setAxisScale(QwtPlot::xBottom,Deb,Fin,TpsMajor);
    ui->qwtPlotHEC->setAxisMaxMajor(QwtPlot::xBottom,NbTpsMed);
    ui->qwtPlotHEC->setAxisMaxMinor(QwtPlot::xBottom,NbTpsMin);

    ui->qwtPlotFEC->setAxisScale(QwtPlot::xBottom,Deb,Fin,TpsMajor);
    ui->qwtPlotFEC->setAxisMaxMajor(QwtPlot::xBottom,NbTpsMed);
    ui->qwtPlotFEC->setAxisMaxMinor(QwtPlot::xBottom,NbTpsMin);

    // échelles Y en fonction du type de courbe
    ui->qwtPlotNoise->setAxisScale(QwtPlot::yLeft,MinNoise,MaxNoise,1);
    ui->qwtPlotNoise->setAxisMaxMinor(QwtPlot::yLeft,10);
    ui->qwtPlotNoise->setAxisMaxMajor(QwtPlot::yLeft,2);

    ui->qwtPlotMaxRate->setAxisScale(QwtPlot::yLeft,MinMaxRate,MaxMaxRate,PasMaxRate);
    ui->qwtPlotMaxRate->setAxisMaxMinor(QwtPlot::yLeft,10);
    ui->qwtPlotMaxRate->setAxisMaxMajor(QwtPlot::yLeft,2);

    ui->qwtPlotCRC->setAxisScale(QwtPlot::yLeft,MinCRC,MaxCRC);
    ui->qwtPlotCRC->setAxisMaxMinor(QwtPlot::yLeft,10);

    ui->qwtPlotHEC->setAxisScale(QwtPlot::yLeft,MinHEC,MaxHEC);
    ui->qwtPlotHEC->setAxisMaxMinor(QwtPlot::yLeft,10);

    ui->qwtPlotFEC->setAxisScale(QwtPlot::yLeft,MinFEC,MaxFEC);
    ui->qwtPlotFEC->setAxisMaxMinor(QwtPlot::yLeft,10);

    // pour régler les largeurs de stylo
    CurveSynchNoise->setPen(Qt::magenta,PenSynch,Qt::DashLine);
    CurveDownNoise->setPen(Qt::blue,CoefPen*PenCurve,Qt::SolidLine);
    CurveUpNoise->setPen(Qt::red,CoefPen*PenCurve,Qt::SolidLine);
    CurveSynchMaxRate->setPen(Qt::magenta,PenSynch,Qt::DashLine);
    CurveDownMaxRate->setPen(Qt::darkBlue,CoefPen*PenCurve,Qt::DotLine);
    CurveUpMaxRate->setPen(Qt::darkRed,CoefPen*PenCurve,Qt::DotLine);
    CurveDownDslRate->setPen(Qt::blue,CoefPen*PenCurve,Qt::SolidLine);
    CurveUpDslRate->setPen(Qt::red,CoefPen*PenCurve,Qt::SolidLine);
    CurveSynchCRC->setPen(Qt::magenta,PenSynch,Qt::DashLine);
    CurveLogCRC->setPen(Qt::blue,CoefPen*PenHisto,Qt::SolidLine);
    CurveLogATUCCRC->setPen(Qt::red,CoefPen*PenHisto,Qt::SolidLine);
    CurveSynchHEC->setPen(Qt::magenta,PenSynch,Qt::DashLine);
    CurveLogHEC->setPen(QColor(80,110,180,255),CoefPen*PenHisto,Qt::SolidLine);
    CurveLogATUCHEC->setPen(QColor(220,40,70,255),CoefPen*PenHisto,Qt::SolidLine); // orange
    CurveSynchFEC->setPen(Qt::magenta,PenSynch,Qt::DashLine);
    CurveLogFEC->setPen(Qt::darkGreen,CoefPen*PenHisto,Qt::SolidLine);
    CurveLogATUCFEC->setPen(QColor(255,192,0,255),CoefPen*PenHisto,Qt::SolidLine); // orange

    // mises à jours
    ui->qwtPlotNoise->replot();
    ui->qwtPlotMaxRate->replot();
    ui->qwtPlotCRC->replot();
    ui->qwtPlotHEC->replot();
    ui->qwtPlotFEC->replot();

}

void Graph::UpDateCurves(LiveBoxInfos *LBox, LiveBoxInfos *OldLBox)
{
    double Temps;
    double OldTps;
    double Log;
    Temps   = QwtDate::toDouble(QDateTime::fromString(LBox->DateTime(),"dd/MM/yyyy hh:mm:ss"));
    OldTps  = QwtDate::toDouble(QDateTime::fromString(OldLBox->DateTime(),"dd/MM/yyyy hh:mm:ss"));
    Dernier = Temps;
    if (OldTps<Debut) Debut=OldTps;                     // le recalage de "début"Debut" est utile dans le cas du chargement d'un CSV
    ui->scrollTemps->setMaximum((Dernier-Debut)/60000);
    ui->scrollTemps->setValue(Offset/60000);

    // marquage des zone d'arrêt
    if ((Temps-OldTps) > (TempsArret*1000)) {
        addZone(OldTps,Temps);
    }

    if (LBox->LastChangeTime() != OldLBox->LastChangeTime()) {
        PointSynch << QPointF(Temps-LBox->LastChange()*1000,100000);    // on enregistre l'heure de synchro (ms)
    }
    CurveSynchNoise->setSamples(PointSynch);
    CurveSynchMaxRate->setSamples(PointSynch);
    CurveSynchCRC->setSamples(PointSynch);
    CurveSynchHEC->setSamples(PointSynch);
    CurveSynchFEC->setSamples(PointSynch);

    PointDownNoise << QPointF(Temps,LBox->DownstreamNoiseMargin()/10);
    CurveDownNoise->setSamples(PointDownNoise);
    PointUpNoise << QPointF(Temps,LBox->UpstreamNoiseMargin()/10);
    CurveUpNoise->setSamples(PointUpNoise);

    PointDownMaxRate << QPointF(Temps,LBox->DownstreamMaxRate());
    CurveDownMaxRate->setSamples(PointDownMaxRate);
    PointUpMaxRate << QPointF(Temps,LBox->UpstreamMaxRate());
    CurveUpMaxRate->setSamples(PointUpMaxRate);

    PointDownDslRate << QPointF(Temps,LBox->DownstreamCurrRate());
    CurveDownDslRate->setSamples(PointDownDslRate);
    PointUpDslRate << QPointF(Temps,LBox->UpstreamCurrRate());
    CurveUpDslRate->setSamples(PointUpDslRate);

    Log=LBox->CRCErrors()-OldLBox->CRCErrors();
    if (Log<=0) Log=0.1;
    PointLogCRC << QPointF(Temps-DecHistoSec*1000,Log);
    CurveLogCRC->setSamples(PointLogCRC);

    Log=LBox->ATUCCRCErrors()-OldLBox->ATUCCRCErrors();
    if (Log<=0) Log=0.1;
    PointLogATUCCRC << QPointF(Temps+DecHistoSec*1000,Log);
    CurveLogATUCCRC->setSamples(PointLogATUCCRC);

    Log=LBox->HECErrors()-OldLBox->HECErrors();
    if (Log<=0) Log=0.1;
    PointLogHEC << QPointF(Temps-DecHistoSec*1000,Log);
    CurveLogHEC->setSamples(PointLogHEC);

    Log=LBox->ATUCHECErrors()-OldLBox->ATUCHECErrors();
    if (Log<=0) Log=0.1;
    PointLogATUCHEC << QPointF(Temps+DecHistoSec*1000,Log);
    CurveLogATUCHEC->setSamples(PointLogATUCHEC);

    Log=LBox->FECErrors()-OldLBox->FECErrors();
    if (Log<=0) Log=0.1;
    PointLogFEC << QPointF(Temps-DecHistoSec*1000,Log);
    CurveLogFEC->setSamples(PointLogFEC);

    Log=LBox->ATUCFECErrors()-OldLBox->ATUCFECErrors();
    if (Log<=0) Log=0.1;
    PointLogATUCFEC << QPointF(Temps+DecHistoSec*1000,Log);
    CurveLogATUCFEC->setSamples(PointLogATUCFEC);

    UpDateScales();             // gère les échelles en particulier celle des temps pour l'avancée

    ui->qwtPlotNoise->replot();
    ui->qwtPlotMaxRate->replot();
    ui->qwtPlotCRC->replot();
    ui->qwtPlotHEC->replot();
    ui->qwtPlotFEC->replot();

}

void Graph::DiapoTimerHandler()
{
    if (DiapoErreurs) {
        if (ui->tabErreurs->currentIndex() < ui->tabErreurs->count()-1) {
            ui->tabErreurs->setCurrentIndex(ui->tabErreurs->currentIndex()+1);
        }
        else {
            ui->tabErreurs->setCurrentIndex(0);
        }
    }
    if (DiapoCourbes) {
        if (ui->tabCourbes->currentIndex() < ui->tabCourbes->count()-1) {
            ui->tabCourbes->setCurrentIndex(ui->tabCourbes->currentIndex()+1);
        }
        else {
            ui->tabCourbes->setCurrentIndex(0);
        }
    }
    DiapoTimer->start(Diapo);
}

void Graph::on_Picker_Temps(QPointF Pos)
{
    int Index=0;
    for (int i=0; i<PointDownNoise.size(); i++) {
        if (Pos.x()>=PointDownNoise[i].x()) Index=i;
    }
    Pos.setX(PointDownNoise[Index].x());
    Pos.setY(10);

    bool SensFlag=true;
    if ( PointDownNoise[Index].x() < ((XtpsMax+XtpsMin)/2) ) SensFlag=false;

    MarkerNoise.setLineStyle(QwtPlotMarker::VLine);
    MarkerNoise.setLinePen(Qt::black,2,Qt::DotLine);
    MarkerNoise.setValue(Pos);
    QwtText qwtLabNoise; // car possibilité d'affecter des fonts et des couleurs plus facilement qu'avec QString
    QString labNoise=QDateTime::fromTime_t(PointDownNoise[Index].x()/1000).toString("dd/MM/yyyy à hh:mm:ss");
    labNoise=labNoise+"\n"+"Down = "+QString::number(PointDownNoise[Index].y(),10,1)+" dB";
    labNoise=labNoise+"\n"+"Up = "+QString::number(PointUpNoise[Index].y(),10,1)+" dB";
    qwtLabNoise.setBackgroundBrush(QBrush(QColor(255,240,170,191))); // fond beige en semi-transparence (128=50% , 191=75%)
    qwtLabNoise.setText(labNoise);
    MarkerNoise.setLabel(qwtLabNoise);
    if (SensFlag) {
        MarkerNoise.setLabelAlignment(Qt::AlignLeft | Qt::AlignTop);
    }
    else {
        MarkerNoise.setLabelAlignment(Qt::AlignRight | Qt::AlignTop);
    }
    MarkerNoise.attach(ui->qwtPlotNoise);

    MarkerMaxRate.setLineStyle(QwtPlotMarker::VLine);
    MarkerMaxRate.setLinePen(Qt::black,2,Qt::DotLine);
    MarkerMaxRate.setValue(Pos);
    QwtText qwtLabMaxRate;
    QString labMaxRate=QDateTime::fromTime_t(PointDownNoise[Index].x()/1000).toString("dd/MM/yyyy à hh:mm:ss");
    labMaxRate=labMaxRate+"\n"+"Down max = "+QString::number(PointDownMaxRate[Index].y(),10,0)+" Kb/s";
    labMaxRate=labMaxRate+"\n"+"Down DSL = "+QString::number(PointDownDslRate[Index].y(),10,0)+" Kb/s";
    labMaxRate=labMaxRate+"\n"+"Up max= "+QString::number(PointUpMaxRate[Index].y(),10,0)+" Kb/s";
    labMaxRate=labMaxRate+"\n"+"Up DSL= "+QString::number(PointUpDslRate[Index].y(),10,0)+" Kb/s";
    qwtLabMaxRate.setBackgroundBrush(QBrush(QColor(255,240,170,191))); // fond beige en semi-transparence (128=50% , 191=75%)
    qwtLabMaxRate.setText(labMaxRate);
    MarkerMaxRate.setLabel(qwtLabMaxRate);
    if (SensFlag) {
        MarkerMaxRate.setLabelAlignment(Qt::AlignLeft | Qt::AlignTop);
    }
    else {
        MarkerMaxRate.setLabelAlignment(Qt::AlignRight | Qt::AlignTop);
    }
    MarkerMaxRate.attach(ui->qwtPlotMaxRate);

    MarkerCRC.setLineStyle(QwtPlotMarker::VLine);
    MarkerCRC.setLinePen(Qt::black,2,Qt::DotLine);
    MarkerCRC.setValue(Pos);
    QwtText qwtLabCRC;
    QString labCRC=QDateTime::fromTime_t(PointDownNoise[Index].x()/1000).toString("dd/MM/yyyy à hh:mm:ss");
    labCRC=labCRC+"\n"+"CRC = "+QString::number(PointLogCRC[Index].y(),10,0);
    labCRC=labCRC+"\n"+"ATU-C CRC = "+QString::number(PointLogATUCCRC[Index].y(),10,0);
    qwtLabCRC.setBackgroundBrush(QBrush(QColor(255,240,170,191))); // fond beige en semi-transparence (128=50% , 191=75%)
    qwtLabCRC.setText(labCRC);
    MarkerCRC.setLabel(qwtLabCRC);
    if (SensFlag) {
        MarkerCRC.setLabelAlignment(Qt::AlignLeft | Qt::AlignTop);
    }
    else {
        MarkerCRC.setLabelAlignment(Qt::AlignRight | Qt::AlignTop);
    }
    MarkerCRC.attach(ui->qwtPlotCRC);

    MarkerHEC.setLineStyle(QwtPlotMarker::VLine);
    MarkerHEC.setLinePen(Qt::black,2,Qt::DotLine);
    MarkerHEC.setValue(Pos);
    QwtText qwtLabHEC;
    QString labHEC=QDateTime::fromTime_t(PointDownNoise[Index].x()/1000).toString("dd/MM/yyyy à hh:mm:ss");
    labHEC=labHEC+"\n"+"HEC = "+QString::number(PointLogHEC[Index].y(),10,0);
    labHEC=labHEC+"\n"+"ATU-C HEC = "+QString::number(PointLogATUCHEC[Index].y(),10,0);
    qwtLabHEC.setBackgroundBrush(QBrush(QColor(255,240,170,191))); // fond beige en semi-transparence (128=50% , 191=75%)
    qwtLabHEC.setText(labHEC);
    MarkerHEC.setLabel(qwtLabHEC);
    if (SensFlag) {
        MarkerHEC.setLabelAlignment(Qt::AlignLeft | Qt::AlignTop);
    }
    else {
        MarkerHEC.setLabelAlignment(Qt::AlignRight | Qt::AlignTop);
    }
    MarkerHEC.attach(ui->qwtPlotHEC);

    MarkerFEC.setLineStyle(QwtPlotMarker::VLine);
    MarkerFEC.setLinePen(Qt::black,2,Qt::DotLine);
    MarkerFEC.setValue(Pos);
    QwtText qwtLabFEC;
    QString labFEC=QDateTime::fromTime_t(PointDownNoise[Index].x()/1000).toString("dd/MM/yyyy à hh:mm:ss");
    labFEC=labFEC+"\n"+"FEC = "+QString::number(PointLogFEC[Index].y(),10,0);
    labFEC=labFEC+"\n"+"ATU-C FEC = "+QString::number(PointLogATUCFEC[Index].y(),10,0);
    qwtLabFEC.setBackgroundBrush(QBrush(QColor(255,240,170,191))); // fond beige en semi-transparence (128=50% , 191=75%)
    qwtLabFEC.setText(labFEC);
    MarkerFEC.setLabel(qwtLabFEC);
    if (SensFlag) {
        MarkerFEC.setLabelAlignment(Qt::AlignLeft | Qt::AlignTop);
    }
    else {
        MarkerFEC.setLabelAlignment(Qt::AlignRight | Qt::AlignTop);
    }
    MarkerFEC.attach(ui->qwtPlotFEC);

    // pour visualiser les lignes de marquage
    ui->qwtPlotNoise->replot();
    ui->qwtPlotMaxRate->replot();
    ui->qwtPlotCRC->replot();
    ui->qwtPlotHEC->replot();
    ui->qwtPlotFEC->replot();
}

void Graph::ResetMarker()
{
    MarkerNoise.detach();
    MarkerMaxRate.detach();
    MarkerCRC.detach();
    MarkerFEC.detach();
    MarkerHEC.detach();

    // pour remettre à jour sans les marquages
    ui->qwtPlotNoise->replot();
    ui->qwtPlotMaxRate->replot();
    ui->qwtPlotCRC->replot();
    ui->qwtPlotHEC->replot();
    ui->qwtPlotFEC->replot();
}

void Graph::on_tabCourbes_currentChanged()
{
    // nécessaire pour visualiser les modifs d'échelles sur les onglets masqués
    ui->qwtPlotNoise->replot();
    ui->qwtPlotMaxRate->replot();
}

void Graph::on_tabErreurs_currentChanged()
{
    // nécessaire pour visualiser les modifs d'échelles sur les onglets masqués
    ui->qwtPlotCRC->replot();
    ui->qwtPlotHEC->replot();
    ui->qwtPlotFEC->replot();
}

//===============================================================
//=============  Gestion des actions sur les boutons

bool Graph::eventFilter(QObject *obj, QEvent *event) // pour intercepter les évènements souris
{
    if (obj->objectName().left(7) == "qwtPlot") {                           // on ne trate que les objets dont le nom commence par "qwtPlot"
        if (event->type() == QEvent::MouseButtonPress) {                    // on ne traite que les boutons souris
            QMouseEvent *mouseEvent = static_cast<QMouseEvent*>(event);
            if (mouseEvent->button() == Qt::RightButton) ResetMarker();     // click droit = effacer les marquages  autrement on laisse passer
            return true;
        } else {
            return false;
        }
    } else {
        // pass the event on to the parent class
        return QDialog::eventFilter(obj, event);
    }
}

void Graph::on_BtnDiapo_toggled(bool checked)
{
    if (checked) {
        DiapoTimer->start(Diapo);
        ui->BtnDiapo->setText("DIAPO OFF");
    }
    else {
        DiapoTimer->stop();
        ui->BtnDiapo->setText("DIAPO ON");
    }
}

void Graph::on_checkAvanceAuto_toggled(bool checked)
{
    ui->scrollTemps->setEnabled(!checked); // si l'avance est automatique on désactive la scroll bar du temps
    UpDateScales();
}

void Graph::on_scrollTemps_valueChanged(int value)
{
    if (!ui->checkAvanceAuto->isChecked()) {
        Offset=value*60000;
        UpDateScales();  // on effectue la mise à jour qui si on est en manuel
    }
}

//--- réglages échelles X -------------------

void Graph::on_BtnZoomPlus_clicked()
{
    if (PlageHeure>1) PlageHeure=PlageHeure-0.5;
    UpDateScales();
}

void Graph::on_BtnZoomMoins_clicked()
{
    if (PlageHeure<TempsMaxHeure-1) PlageHeure=PlageHeure+0.5;
    UpDateScales();
}

void Graph::on_BtnZoomTout_clicked()
{
    double Pl = (Dernier-Debut)/(3600*1000); // durée de relevé transformée en heures
    if (Pl<0.5) Pl=0.5;                      // minimum 1/2H
    PlageHeure = Pl + 2*ReculMinute/60;      // on ajoute l'intervalle au début et à la fin
    ui->checkAvanceAuto->setChecked(true);
    UpDateScales();
}

void Graph::on_BtnZoom3H_clicked()
{
    PlageHeure=3;
    ui->checkAvanceAuto->setChecked(true);
    UpDateScales();
}

//--- réglages échelles Y -------------------

void Graph::on_BtnNoiseMaxPlus_clicked()
{
    MaxNoise=MaxNoise+1;
    UpDateScales();
}

void Graph::on_BtnNoiseMinMoins_clicked()
{
    if (MinNoise>0) MinNoise=MinNoise-1;
    UpDateScales();
}

void Graph::on_BtnNoiseMaxMoins_clicked()
{
    if (MaxNoise>(MinNoise+3)) MaxNoise=MaxNoise-1;
    UpDateScales();
}

void Graph::on_BtnNoiseMinPlus_clicked()
{
    if (MaxNoise>(MinNoise+3)) MinNoise=MinNoise+1;
    UpDateScales();
}

void Graph::on_BtnMaxRateMaxPlus_clicked()
{
    MaxMaxRate=MaxMaxRate+PasMaxRate;
    UpDateScales();
}

void Graph::on_BtnMaxRateMaxMoins_clicked()
{
    if (MaxMaxRate>(MinMaxRate+3*PasMaxRate)) MaxMaxRate=MaxMaxRate-PasMaxRate;
    UpDateScales();
}

void Graph::on_BtnMaxRateMinPlus_clicked()
{
    if (MaxMaxRate>(MinMaxRate+3*PasMaxRate)) MinMaxRate=MinMaxRate+PasMaxRate;
    UpDateScales();
}

void Graph::on_BtnMaxRateMinMoins_clicked()
{
    if (MinMaxRate>0) MinMaxRate=MinMaxRate-PasMaxRate;
    UpDateScales();
}

void Graph::on_BtnFECMaxPlus_clicked()
{
    if (MaxFEC>=10000) {
        MaxFEC=MaxFEC+10000;
    }
    else {
        if (MaxFEC>=1000) MaxFEC=MaxFEC+1000;
    }
    UpDateScales();
}

void Graph::on_BtnFECMaxMoins_clicked()
{
    if (MaxFEC>=20000) {
        MaxFEC=MaxFEC-10000;
    }
    else {
        if (MaxFEC>=2000) MaxFEC=MaxFEC-1000;
    }
    UpDateScales();
}

void Graph::on_BtnCRCMaxPlus_clicked()
{
    if (MaxCRC >=100) {
        MaxCRC=MaxCRC+100;
    }
    else {
        if (MaxCRC>=50) MaxCRC=MaxCRC+10;
    }
    UpDateScales();
}

void Graph::on_BtnCRCMaxMoins_clicked()
{
    if (MaxCRC >=200) {
        MaxCRC=MaxCRC-100;
    }
    else {
        if (MaxCRC>=60) MaxCRC=MaxCRC-10;
    }
    UpDateScales();
}

void Graph::on_BtnHECMaxPlus_clicked()
{
    if (MaxHEC >=100) {
        MaxHEC=MaxHEC+100;
    }
    else {
        if (MaxHEC>=50) MaxHEC=MaxHEC+10;
    }
    UpDateScales();
}

void Graph::on_BtnHECMaxMoins_clicked()
{
    if (MaxHEC >=200) {
        MaxHEC=MaxHEC-100;
    }
    else {
        if (MaxHEC>=60) MaxHEC=MaxHEC-10;
    }
    UpDateScales();
}

void Graph::on_BtnExportNoise_clicked()
{
    QString FileExport;
    FileExport=QApplication::applicationDirPath()+"/export/"+SSID+"_Marges_"+QDateTime::currentDateTime().toString("_yyyyMMdd_HHmmss")+"."+ExportFormat;
    QwtPlotRenderer renderer;
    ui->qwtPlotNoise->setTitle(SSID+" : Marges de bruit (db)");
    renderer.renderDocument(ui->qwtPlotNoise,FileExport,ExportFormat,QSizeF(ui->qwtPlotNoise->width()/ExportScale,ui->qwtPlotNoise->height()/ExportScale),ExportResol);
    ui->qwtPlotNoise->setTitle("");
    QString msg;
    msg="Exportation du fichier\n"+FileExport+"\n"+"terminée.";
    QMessageBox::information(this,"Export marges de bruits",msg);
}

void Graph::on_BtnExportFEC_clicked()
{
    QString FileExport;
    FileExport=QApplication::applicationDirPath()+"/export/"+SSID+"_FEC_"+QDateTime::currentDateTime().toString("_yyyyMMdd_HHmmss")+"."+ExportFormat;
    QwtPlotRenderer renderer;
    ui->qwtPlotFEC->setTitle(SSID+" : Nombre de FEC et ATU-C FEC");
    renderer.renderDocument( ui->qwtPlotFEC, FileExport,ExportFormat,QSizeF(ui->qwtPlotFEC->width()/ExportScale,ui->qwtPlotFEC->height()/ExportScale),ExportResol);
    ui->qwtPlotFEC->setTitle("");
    QString msg;
    msg="Exportation du fichier\n"+FileExport+"\n"+"terminée.";
    QMessageBox::information(this,"Export nombre de FEC",msg);
}

void Graph::on_BtnExportHEC_clicked()
{
    QString FileExport;
    FileExport=QApplication::applicationDirPath()+"/export/"+SSID+"_HEC_"+QDateTime::currentDateTime().toString("_yyyyMMdd_HHmmss")+"."+ExportFormat;
    QwtPlotRenderer renderer;
    ui->qwtPlotHEC->setTitle(SSID+" : Nombre de HEC et ATU-C HEC");
    renderer.renderDocument( ui->qwtPlotHEC, FileExport,ExportFormat,QSizeF(ui->qwtPlotHEC->width()/ExportScale,ui->qwtPlotHEC->height()/ExportScale),ExportResol);
    ui->qwtPlotHEC->setTitle("");
    QString msg;
    msg="Exportation du fichier\n"+FileExport+"\n"+"terminée.";
    QMessageBox::information(this,"Export nombre de HEC",msg);
}

void Graph::on_BtnExportCRC_clicked()
{
    QString FileExport;
    FileExport=QApplication::applicationDirPath()+"/export/"+SSID+"_CRC_"+QDateTime::currentDateTime().toString("_yyyyMMdd_HHmmss")+"."+ExportFormat;
    QwtPlotRenderer renderer;
    ui->qwtPlotCRC->setTitle(SSID+" : Nombre de CRC et ATU-C CRC");
    renderer.renderDocument( ui->qwtPlotCRC, FileExport,ExportFormat,QSizeF(ui->qwtPlotCRC->width()/ExportScale,ui->qwtPlotCRC->height()/ExportScale),ExportResol);
    ui->qwtPlotCRC->setTitle("");
    QString msg;
    msg="Exportation du fichier\n"+FileExport+"\n"+"terminée.";
    QMessageBox::information(this,"Export nombre de CRC",msg);
}

void Graph::on_BtnExportMaxRate_clicked()
{
    QString FileExport;
    FileExport=QApplication::applicationDirPath()+"/export/"+SSID+"_Debits_Maxis_"+QDateTime::currentDateTime().toString("_yyyyMMdd_HHmmss")+"."+ExportFormat;
    QwtPlotRenderer renderer;
    ui->qwtPlotMaxRate->setTitle(SSID+" : Débits maxis (Kb/s)");
    renderer.renderDocument( ui->qwtPlotMaxRate, FileExport,ExportFormat,QSizeF(ui->qwtPlotMaxRate->width()/ExportScale,ui->qwtPlotMaxRate->height()/ExportScale),ExportResol);
    ui->qwtPlotMaxRate->setTitle("");
    QString msg;
    msg="Exportation du fichier\n"+FileExport+"\n"+"terminée.";
    QMessageBox::information(this,"Export débits maxis",msg);
}

void Graph::RapportGraphique(QString File, QString Day)
{
    // pour mettre a jour les affichages et hauteurs
    int onglet=ui->tabCourbes->currentIndex();
    for (int i=0; i<ui->tabCourbes->count();i++) ui->tabCourbes->setCurrentIndex(i);
    ui->tabCourbes->setCurrentIndex(onglet);
    onglet=ui->tabErreurs->currentIndex();
    for (int i=0; i<ui->tabErreurs->count();i++) ui->tabErreurs->setCurrentIndex(i);
    ui->tabErreurs->setCurrentIndex(onglet);

    QString FileName;
    QString ToDay;
    if (File=="") {
        QString Path=QApplication::applicationDirPath()+"/export/";
        QString Name=SSID+"_Rapport"+QDateTime::currentDateTime().toString("_yyyyMMdd_HHmmss")+".pdf";
        FileName=QFileDialog::getSaveFileName(this,"Enregistrer le rapport PDF",Path+Name,"Fichiers PDF (*.pdf);;tous (*.*)");
        ToDay = QDateTime::currentDateTime().toString("dd/MM/yyyy");
    }
    else {
        FileName=File;
        ToDay=Day;
    }
    QString Titre;
    Titre=SSID+"   -   "+ToDay;

    if (FileName=="") return;

    int ResolPrint=200;
    double Largeur;
    double Hauteur;
    int    HtTitrePage  = 50;
    int    HtTitreGraph = 40;

    QPrinter printer;
    printer.setOutputFormat(QPrinter::PdfFormat);
    printer.setPaperSize(QPrinter::A4);
    printer.setResolution(ResolPrint);

    qDebug() << "RapPageOrient" << RapPageOrient;
    if (RapPageOrient==1) {
        qDebug() << "Paysage";
        printer.setOrientation(QPrinter::Landscape);
        Largeur = 11.5 * ResolPrint;
        Hauteur =  8   * ResolPrint;
    }
    else {
        qDebug() << "Portrait";
        printer.setOrientation(QPrinter::Portrait);
        Hauteur = 11.5 * ResolPrint;
        Largeur =  8   * ResolPrint;
    }
    printer.setOutputFileName(FileName);
    QPainter painter(&printer);
    QRectF Page = QRectF(0,0,Largeur,Hauteur);

    // calcul des hauteurs des graphiques
    double HtPage  = Hauteur-HtTitrePage;
    double HtGraph = 0;
    if (ui->checkNoisePdf->isChecked()) {
        HtGraph = HtGraph + ui->qwtPlotNoise->height();
        qDebug() << "Noise Ht" << ui->qwtPlotNoise->height();
        HtPage  = HtPage  - HtTitreGraph;
    }
    if (ui->checkDebitMaxPdf->isChecked()) {
        qDebug() << "Debit Ht" << ui->qwtPlotMaxRate->height();
        HtGraph = HtGraph + ui->qwtPlotMaxRate->height();
        HtPage  = HtPage  - HtTitreGraph;
    }
    if (ui->checkCrcPdf->isChecked()) {
        qDebug() << "CRC Ht" << ui->qwtPlotCRC->height();
        HtGraph = HtGraph + ui->qwtPlotCRC->height();
        HtPage  = HtPage  - HtTitreGraph;
    }
    if (ui->checkHecPdf->isChecked()) {
        qDebug() << "HEC Ht" << ui->qwtPlotHEC->height();
        HtGraph = HtGraph + ui->qwtPlotHEC->height();
        HtPage  = HtPage  - HtTitreGraph;
    }
    if (ui->checkFecPdf->isChecked()) {
        qDebug() << "FEC Ht" << ui->qwtPlotFEC->height();
        HtGraph = HtGraph + ui->qwtPlotFEC->height();
        HtPage  = HtPage  - HtTitreGraph;
    }
    qDebug() << "Page" << HtPage << "Graph" << HtGraph;
    double Coef = HtPage / HtGraph; // pour recalculer les hauteurs de graphiques
    qDebug() << "Coef" << Coef;
    int    Ht=0;
    int    Top=0;

    // préparation de la page
    QFont font=painter.font();
    font.setPointSize(12);
    font.setBold(true);
    font.setItalic(true);
    painter.setFont(font);
    painter.drawText(QRect(0,0,Page.width(),HtTitrePage),Qt::AlignCenter,Titre);
    Top=HtTitrePage;

    font.setPointSize(6);
    font.setBold(false);
    font.setItalic(true);
    painter.setFont(font);
    Titre="Type : "+HardwareVersion+" ; software : "+SoftwareVersion;
    painter.drawText(QRect(0,5,Page.width()-15,HtTitrePage),Qt::AlignRight,Titre);

    font.setPointSize(8);
    font.setBold(true);
    font.setItalic(false);
    painter.setFont(font);

    QwtPlotRenderer renderer;
    QBrush BackGround;
    QString plotStyle;
    QPalette paletPlot;

    QPalette paletPage=ui->qwtPlotNoise->palette();
    paletPage.setBrush(QPalette::Window,Qt::white);


    if (ui->checkNoisePdf->isChecked()) {
        Titre = "Marges de bruit upload et download (dB)";
        painter.drawText(QRect(0,Top,Page.width(),HtTitreGraph),Qt::AlignLeft|Qt::AlignBottom,Titre);
        Top = Top + HtTitreGraph;
        Ht= Coef * ui->qwtPlotNoise->height();
        BackGround  = ui->qwtPlotNoise->canvasBackground();
        plotStyle   = ui->qwtPlotNoise->styleSheet();
        paletPlot   = ui->qwtPlotNoise->palette();
        ui->qwtPlotNoise->setPalette(paletPage);
        ui->qwtPlotNoise->setCanvasBackground(QColor(Qt::white));
        renderer.render( ui->qwtPlotNoise, &painter,QRectF( Page.left(), Top, Page.width(), Ht ) );
        Top=Top+Ht;
        ui->qwtPlotNoise->setTitle("");
        ui->qwtPlotNoise->setPalette(paletPlot);
        ui->qwtPlotNoise->setCanvasBackground(BackGround);
    }

    if (ui->checkDebitMaxPdf->isChecked()) {
        Titre = "Débits maxis upload et download (kB/s)";
        painter.drawText(QRect(0,Top,Page.width(),HtTitreGraph),Qt::AlignLeft|Qt::AlignBottom,Titre);
        Top = Top + HtTitreGraph;
        Ht= Coef * ui->qwtPlotMaxRate->height();
        BackGround  = ui->qwtPlotMaxRate->canvasBackground();
        plotStyle   = ui->qwtPlotMaxRate->styleSheet();
        paletPlot   = ui->qwtPlotMaxRate->palette();
        ui->qwtPlotMaxRate->setPalette(paletPage);
        ui->qwtPlotMaxRate->setCanvasBackground(QColor(Qt::white));
        renderer.render( ui->qwtPlotMaxRate, &painter,QRectF( Page.left(), Top, Page.width(), Ht ) );
        Top=Top+Ht;
        ui->qwtPlotMaxRate->setTitle("");
        ui->qwtPlotMaxRate->setPalette(paletPlot);
        ui->qwtPlotMaxRate->setCanvasBackground(BackGround);
    }

    if (ui->checkCrcPdf->isChecked()) {
        Titre = "Nombre d'erreurs CRC et ATU-C CRC";
        painter.drawText(QRect(0,Top,Page.width(),HtTitreGraph),Qt::AlignLeft|Qt::AlignBottom,Titre);
        Top = Top + HtTitreGraph;
        Ht= Coef * ui->qwtPlotCRC->height();
        BackGround  = ui->qwtPlotCRC->canvasBackground();
        plotStyle   = ui->qwtPlotCRC->styleSheet();
        paletPlot   = ui->qwtPlotCRC->palette();
        ui->qwtPlotCRC->setPalette(paletPage);
        ui->qwtPlotCRC->setCanvasBackground(QColor(Qt::white));
        renderer.render( ui->qwtPlotCRC, &painter,QRectF( Page.left(), Top, Page.width(), Ht ) );
        Top=Top+Ht;
        ui->qwtPlotCRC->setTitle("");
        ui->qwtPlotCRC->setPalette(paletPlot);
        ui->qwtPlotCRC->setCanvasBackground(BackGround);
    }

    if (ui->checkHecPdf->isChecked()) {
        Titre = "Nombre d'erreurs HEC et ATU-C HEC";
        painter.drawText(QRect(0,Top,Page.width(),HtTitreGraph),Qt::AlignLeft|Qt::AlignBottom,Titre);
        Top = Top + HtTitreGraph;
        Ht= Coef * ui->qwtPlotHEC->height();
        BackGround  = ui->qwtPlotHEC->canvasBackground();
        plotStyle   = ui->qwtPlotHEC->styleSheet();
        paletPlot   = ui->qwtPlotHEC->palette();
        ui->qwtPlotHEC->setPalette(paletPage);
        ui->qwtPlotHEC->setCanvasBackground(QColor(Qt::white));
        renderer.render( ui->qwtPlotHEC, &painter,QRectF( Page.left(), Top, Page.width(), Ht ) );
        Top=Top+Ht;
        ui->qwtPlotHEC->setTitle("");
        ui->qwtPlotHEC->setPalette(paletPlot);
        ui->qwtPlotHEC->setCanvasBackground(BackGround);
    }

    if (ui->checkFecPdf->isChecked()) {
        Titre = "Nombre d'erreurs FEC et ATU-C FEC";
        painter.drawText(QRect(0,Top,Page.width(),HtTitreGraph),Qt::AlignLeft|Qt::AlignBottom,Titre);
        Top = Top + HtTitreGraph;
        Ht= Coef * ui->qwtPlotFEC->height();
        BackGround  = ui->qwtPlotFEC->canvasBackground();
        plotStyle   = ui->qwtPlotFEC->styleSheet();
        paletPlot   = ui->qwtPlotFEC->palette();
        ui->qwtPlotFEC->setPalette(paletPage);
        ui->qwtPlotFEC->setCanvasBackground(QColor(Qt::white));
        renderer.render( ui->qwtPlotFEC, &painter,QRectF( Page.left(), Top, Page.width(), Ht ) );
        Top=Top+Ht;
        ui->qwtPlotFEC->setTitle("");
        ui->qwtPlotFEC->setPalette(paletPlot);
        ui->qwtPlotFEC->setCanvasBackground(BackGround);
    }

    painter.drawRect(Page);
}

void Graph::addZone(double Deb, double Fin)
{
    QColor c = QColor(255,80,80,255);
    QColor b = QColor(255,80,80,50);

    QwtPlotZoneItem *ZoneN = new QwtPlotZoneItem;
    ListZone.append(ZoneN);
    ZoneN->setOrientation(Qt::Vertical);
    ZoneN->setPen(c);
    ZoneN->setBrush(b);
    ZoneN->setInterval(Deb,Fin);
    ZoneN->attach(ui->qwtPlotNoise);

    QwtPlotZoneItem *ZoneC = new QwtPlotZoneItem;
    ListZone.append(ZoneC);
    ZoneC->setOrientation(Qt::Vertical);
    ZoneC->setPen(c);
    ZoneC->setBrush(b);
    ZoneC->setInterval(Deb,Fin);
    ZoneC->attach(ui->qwtPlotCRC);

    QwtPlotZoneItem *ZoneH = new QwtPlotZoneItem;
    ListZone.append(ZoneH);
    ZoneH->setOrientation(Qt::Vertical);
    ZoneH->setPen(c);
    ZoneH->setBrush(b);
    ZoneH->setInterval(Deb,Fin);
    ZoneH->attach(ui->qwtPlotHEC);

    QwtPlotZoneItem *ZoneF = new QwtPlotZoneItem;
    ListZone.append(ZoneF);
    ZoneF->setOrientation(Qt::Vertical);
    ZoneF->setPen(c);
    ZoneF->setBrush(b);
    ZoneF->setInterval(Deb,Fin);
    ZoneF->attach(ui->qwtPlotFEC);

    QwtPlotZoneItem *ZoneM = new QwtPlotZoneItem;
    ListZone.append(ZoneM);
    ZoneM->setOrientation(Qt::Vertical);
    ZoneM->setPen(c);
    ZoneM->setBrush(b);
    ZoneM->setInterval(Deb,Fin);
    ZoneM->attach(ui->qwtPlotMaxRate);

}

void Graph::clearListZone() // pour purger les zones lors d'un RAZ et libérer la mémoire
{
    for (QList<QwtPlotZoneItem*>::iterator it = ListZone.begin(); it != ListZone.end(); ++it)
        delete *it;
    ListZone.clear();
}

void Graph::ClearCurves()
{
    QDateTime qdtNow=QDateTime::currentDateTime();
    QwtDate qwtNow;
    double dNow=qwtNow.toDouble(qdtNow);
    Debut       =   dNow;                               // sera la base quand le début de feuille sera plein

    clearListZone();

    PointDownMaxRate.clear();
    PointDownNoise.clear();
    PointUpMaxRate.clear();
    PointUpNoise.clear();
    PointUpDslRate.clear();
    PointDownDslRate.clear();
    PointLogATUCCRC.clear();
    PointLogATUCFEC.clear();
    PointLogATUCHEC.clear();
    PointLogCRC.clear();
    PointLogFEC.clear();
    PointLogHEC.clear();
    PointSynch.clear();

    CurveDownMaxRate->setSamples(PointDownMaxRate);
    CurveDownNoise->setSamples(PointDownNoise);
    CurveDownDslRate->setSamples(PointDownDslRate);
    CurveUpMaxRate->setSamples(PointUpMaxRate);
    CurveUpNoise->setSamples(PointUpNoise);
    CurveUpDslRate->setSamples(PointUpDslRate);
    CurveLogATUCCRC->setSamples(PointLogATUCCRC);
    CurveLogATUCFEC->setSamples(PointLogATUCFEC);
    CurveLogATUCHEC->setSamples(PointLogATUCHEC);
    CurveLogCRC->setSamples(PointLogCRC);
    CurveLogFEC->setSamples(PointLogFEC);
    CurveLogHEC->setSamples(PointLogHEC);
    CurveSynchCRC->setSamples(PointSynch);
    CurveSynchFEC->setSamples(PointSynch);
    CurveSynchHEC->setSamples(PointSynch);
    CurveSynchMaxRate->setSamples(PointSynch);
    CurveSynchNoise->setSamples(PointSynch);

    // pour remettre à jour les graphiques
    ui->qwtPlotNoise->replot();
    ui->qwtPlotMaxRate->replot();
    ui->qwtPlotCRC->replot();
    ui->qwtPlotHEC->replot();
    ui->qwtPlotFEC->replot();
}

void Graph::InitTabSynchronisations()
{
    // pour l'axe horizontal avec les heures
    QString Hformat="hh";
    qwtScaleXsynchro = new QwtDateScaleDraw();
    qwtScaleXsynchro->setDateFormat(QwtDate::Millisecond,Hformat);
    qwtScaleXsynchro->setDateFormat(QwtDate::Second,Hformat);
    qwtScaleXsynchro->setDateFormat(QwtDate::Minute,Hformat);
    qwtScaleXsynchro->setDateFormat(QwtDate::Hour,Hformat);
    qwtScaleXsynchro->setDateFormat(QwtDate::Day,Hformat);
    qwtScaleXsynchro->setDateFormat(QwtDate::Month,Hformat);
    qwtScaleXsynchro->setDateFormat(QwtDate::Year,Hformat);
    qwtScaleXsynchro->setLabelAlignment(Qt::AlignHCenter);
    // pour l'axe vertical avec les jours
    QString Jformat="dd / MM";
    qwtScaleYsynchro = new QwtDateScaleDraw();
    qwtScaleYsynchro->setDateFormat(QwtDate::Millisecond,Jformat);
    qwtScaleYsynchro->setDateFormat(QwtDate::Second,Jformat);
    qwtScaleYsynchro->setDateFormat(QwtDate::Minute,Jformat);
    qwtScaleYsynchro->setDateFormat(QwtDate::Hour,Jformat);
    qwtScaleYsynchro->setDateFormat(QwtDate::Day,Jformat);
    qwtScaleYsynchro->setDateFormat(QwtDate::Month,Jformat);
    qwtScaleYsynchro->setDateFormat(QwtDate::Year,Jformat);
    qwtScaleYsynchro->setLabelAlignment(Qt::AlignLeft);

    ui->qwtPlotSynchronisations->insertLegend(new QwtLegend(), QwtPlot::RightLegend);
    // Grille

    GridSynchronisations = new QwtPlotGrid();
    GridSynchronisations->setMajorPen( Qt::darkGray, 0.0, Qt::SolidLine );
    GridSynchronisations->setMinorPen( Qt::lightGray, 0.0, Qt::SolidLine);
    GridSynchronisations->enableX(true);
    GridSynchronisations->enableY(true);
    GridSynchronisations->enableXMin(true);
    GridSynchronisations->enableYMin(false);
    GridSynchronisations->attach(ui->qwtPlotSynchronisations);

    CurveConnect = new QwtPlotCurve();
    CurveConnect->setYAxis(QwtPlot::yLeft);
    CurveConnect->setPen(Qt::red,8,Qt::SolidLine);
    CurveConnect->setSymbol( new QwtSymbol( QwtSymbol::XCross, Qt::green,
                                            QPen( Qt::blue , 2), QSize( 14, 14 ) ) );
    CurveConnect->setStyle(QwtPlotCurve::NoCurve);
    CurveConnect->setTitle("Connexion");

    CurveSynchronisations = new QwtPlotCurve();
    CurveSynchronisations->setYAxis(QwtPlot::yLeft);
    CurveSynchronisations->setPen(Qt::red,8,Qt::SolidLine);
    CurveSynchronisations->setSymbol( new QwtSymbol( QwtSymbol::Cross, Qt::green,
                                            QPen( Qt::red , 2), QSize( 14, 14 ) ) );
    CurveSynchronisations->setStyle(QwtPlotCurve::NoCurve);
    CurveSynchronisations->setTitle("Synchro DSL");

    CurveReboot = new QwtPlotCurve();
    CurveReboot->setYAxis(QwtPlot::yLeft);
    CurveReboot->setPen(Qt::green,8,Qt::SolidLine);
    CurveReboot->setSymbol( new QwtSymbol( QwtSymbol::Ellipse, Qt::green,
                                            QPen( Qt::green , 2), QSize( 16, 16 ) ) );
    CurveReboot->setStyle(QwtPlotCurve::NoCurve);
    CurveReboot->setTitle("Reboot box");

    CurveReboot->attach(ui->qwtPlotSynchronisations);
    CurveConnect->attach(ui->qwtPlotSynchronisations);
    CurveSynchronisations->attach(ui->qwtPlotSynchronisations);
    UpDateSynchronisations();

    ui->qwtPlotSynchronisations->replot();
}

void Graph::UpDateSynchronisations()
{
    // Echelles
    QDateTime Now=QDateTime::currentDateTime();
    double Jmax=QwtDate::toDouble(Now)+3*3600*1000;
    double Jmin=Jmax-NbJrSynch*24*3600*1000;
    ui->qwtPlotSynchronisations->enableAxis(QwtPlot::yLeft,true);
    ui->qwtPlotSynchronisations->setAxisScale(QwtPlot::yLeft,Jmin,Jmax,24*3600*1000);
    ui->qwtPlotSynchronisations->setAxisMaxMinor(QwtPlot::yLeft,0);
    ui->qwtPlotSynchronisations->setAxisMaxMajor(QwtPlot::yLeft,0);
    ui->qwtPlotSynchronisations->setAxisScaleDraw(QwtPlot::yLeft,qwtScaleYsynchro);

    ui->qwtPlotSynchronisations->setAxisScale(QwtPlot::xBottom,0,24,1);
    ui->qwtPlotSynchronisations->setAxisMaxMajor(QwtPlot::xBottom,1);
    ui->qwtPlotSynchronisations->setAxisMaxMinor(QwtPlot::xBottom,4);

    PointSynchronisations.clear();
    PointReboot.clear();
    PointConnect.clear();

    // lecture du fichier CSV de stockage des dates et heures de synchronisations
    QString FileName;
    FileName=QApplication::applicationDirPath()+"/"+SSID+"_SYNCHROS.csv";
    //qDebug() << FileName;
    QString     ligne;
    QFile       fichier(FileName);
    if(fichier.open(QIODevice::ReadOnly | QIODevice::Text))
    {
        QTextStream flux(&fichier);
        if (!flux.atEnd()) ligne = flux.readLine(); // pour sauter la ligne d'entête
        while(!flux.atEnd())
        {
            ligne = flux.readLine();
            QStringList listLigne=ligne.split(";");
            //qDebug() << listLigne[0] << listLigne[1] << listLigne[2] << listLigne[3];
            QDateTime dSynch;
            dSynch = QDateTime::fromString(listLigne[0],"dd/MM/yyyy hh:mm:ss");
            double dJour=dSynch.toMSecsSinceEpoch();
            double dh=dSynch.time().hour();
            double dm=dSynch.time().minute();
            double ds=dSynch.time().second();
            double dHeure=dh+dm/60+ds/3600;         // heure en centièmes d'heure
            //qDebug() << "Heure=" << dHeure << "Jour" << dJour;
            PointSynchronisations.append(QPointF(dHeure,dJour));
            QDateTime dReboot;
            dReboot = QDateTime::fromString(listLigne[3],"dd/MM/yyyy hh:mm:ss");
            dJour=dReboot.toMSecsSinceEpoch();
            dh=dReboot.time().hour();
            dm=dReboot.time().minute();
            ds=dReboot.time().second();
            dHeure=dh+dm/60+ds/3600;         // heure en centièmes d'heure
            PointReboot.append(QPointF(dHeure,dJour));
            if (listLigne.count()>4) {          // on test la présence de la données car elle n'est pas présente sur V2.10 et avant
                QDateTime dConnect;
                dConnect = QDateTime::fromString(listLigne[4],"dd/MM/yyyy hh:mm:ss");
                dJour=dConnect.toMSecsSinceEpoch();
                dh=dConnect.time().hour();
                dm=dConnect.time().minute();
                ds=dConnect.time().second();
                dHeure=dh+dm/60+ds/3600;         // heure en centièmes d'heure
                PointConnect.append(QPointF(dHeure,dJour));
            }

        }
        fichier.close();
    }

    CurveSynchronisations->setSamples(PointSynchronisations);
    CurveReboot->setSamples(PointReboot);
    CurveConnect->setSamples(PointConnect);

    ui->qwtPlotSynchronisations->replot();
}

void Graph::on_BtnSynchroPlus_clicked()
{
    NbJrSynch=NbJrSynch+1;
    UpDateSynchronisations();
}

void Graph::on_BtnSynchroMoins_clicked()
{
    if (NbJrSynch>5) NbJrSynch=NbJrSynch-1;
    UpDateSynchronisations();
}

void Graph::on_BtnExportSynchro_clicked()
{
    QString FileExport;
    FileExport=QApplication::applicationDirPath()+"/export/"+SSID+"_Synchronisations_"+QDateTime::currentDateTime().toString("_yyyyMMdd_HHmmss")+"."+ExportFormat;
    QwtPlotRenderer renderer;
    ui->qwtPlotSynchronisations->setTitle(SSID+" : Synthèse des synchronisations DSL");
    renderer.renderDocument( ui->qwtPlotSynchronisations, FileExport,ExportFormat,QSizeF(ui->qwtPlotSynchronisations->width()/ExportScale,ui->qwtPlotSynchronisations->height()/ExportScale),ExportResol);
    ui->qwtPlotSynchronisations->setTitle("");
    QString msg;
    msg="Exportation du fichier\n"+FileExport+"\n"+"terminée.";
    QMessageBox::information(this,"Export synchronisations",msg);

}

void Graph::on_BtnSynchroPdf_clicked()
{
    QString Path=QApplication::applicationDirPath()+"/export/";
    QString Name=SSID+"_Synchronisations"+QDateTime::currentDateTime().toString("_yyyyMMdd_HHmmss")+".pdf";
    QString FileName=QFileDialog::getSaveFileName(this,"Enregistrer le rapport PDF",Path+Name,"Fichiers PDF (*.pdf);;tous (*.*)");
    QString ToDay = QDateTime::currentDateTime().toString("dd/MM/yyyy");

    QString Titre;
    Titre=SSID+"   -   "+ToDay;

    if (FileName=="") return;

    int ResolPrint=200;
    double Largeur;
    double Hauteur;
    int    HtTitrePage  = 50;
    int    HtTitreGraph = 50;

    QPrinter printer;
    printer.setOutputFormat(QPrinter::PdfFormat);
    printer.setPaperSize(QPrinter::A4);
    printer.setResolution(ResolPrint);

    qDebug() << "RapPageOrient" << RapPageOrient;
    if (RapPageOrient==1) {
        qDebug() << "Paysage";
        printer.setOrientation(QPrinter::Landscape);
        Largeur = 11.5 * ResolPrint;
        Hauteur =  8   * ResolPrint;
    }
    else {
        qDebug() << "Portrait";
        printer.setOrientation(QPrinter::Portrait);
        Hauteur = 11.5 * ResolPrint;
        Largeur =  8   * ResolPrint;
    }
    printer.setOutputFileName(FileName);
    QPainter painter(&printer);
    QRectF Page = QRectF(0,0,Largeur,Hauteur);

    int Ht=0;
    int Top=0;

    // préparation de la page
    QFont font=painter.font();
    font.setPointSize(12);
    font.setBold(true);
    font.setItalic(true);
    painter.setFont(font);
    painter.drawText(QRect(0,0,Page.width(),HtTitrePage),Qt::AlignCenter,Titre);

    font.setPointSize(6);
    font.setBold(false);
    font.setItalic(true);
    painter.setFont(font);
    Titre="Type : "+HardwareVersion+" ; software : "+SoftwareVersion;
    painter.drawText(QRect(0,5,Page.width()-15,HtTitrePage),Qt::AlignRight,Titre);

    Top=HtTitrePage;
    font.setPointSize(10);
    font.setItalic(false);
    painter.setFont(font);

    QwtPlotRenderer renderer;
    QBrush BackGround;
    QString plotStyle;
    QPalette paletPlot;

    QPalette paletPage=ui->qwtPlotSynchronisations->palette();
    paletPage.setBrush(QPalette::Window,Qt::white);

    Titre = "Synthèse des synchronisations DSL et des redémarrages livebox.";
    painter.drawText(QRect(0,Top,Page.width(),HtTitreGraph),Qt::AlignCenter,Titre);
    Top = Top + HtTitreGraph;
    Ht= Hauteur-Top;
    BackGround  = ui->qwtPlotSynchronisations->canvasBackground();
    plotStyle   = ui->qwtPlotSynchronisations->styleSheet();
    paletPlot   = ui->qwtPlotSynchronisations->palette();
    ui->qwtPlotSynchronisations->setPalette(paletPage);
    ui->qwtPlotSynchronisations->setCanvasBackground(QColor(Qt::white));
    renderer.render( ui->qwtPlotSynchronisations, &painter,QRectF( Page.left(), Top, Page.width(), Ht ) );
    ui->qwtPlotSynchronisations->setTitle("");
    ui->qwtPlotSynchronisations->setPalette(paletPlot);
    ui->qwtPlotSynchronisations->setCanvasBackground(BackGround);

    painter.drawRect(Page);
}

void Graph::on_BtnUpDateSynchro_clicked()
{
    UpDateSynchronisations();
}

void Graph::SetZoomTout()
{
    on_BtnZoomTout_clicked();
}

void Graph::SetZoom3H()
{
    on_BtnZoom3H_clicked();
}

void Graph::SetZoomPlage(double value)
{
    PlageHeure=value;
    ui->checkAvanceAuto->setChecked(true);
    UpDateScales();
}
